import scalajs.js.annotation._

@JSExport
object Main {

  import scala.io.StdIn
  import org.scalajs.dom._
  import scala.scalajs.js.timers._

  private var state: State= new State()
  private var input: Option[Input]= None // a poor man's mvar
  private var exitHandlers: Unit=> Unit = (_ => ())

  /**
   * Sets up the input handling; returns a  function which cancels it.
   * Stores the parsed input code in global variable input.
   */
  def setInputHandler(): (Unit=> Unit) = {     
     lazy val kdHandler: scalajs.js.Function1[Event,Unit] = { (e: Event) =>
       val event = e.asInstanceOf[KeyboardEvent]
       Input.parseKeycode(event.keyCode).foreach { c => input= Some(c) }
     }
     lazy val kuHandler: scalajs.js.Function1[Event,Unit] = { (e: Event) =>
       val event = e.asInstanceOf[KeyboardEvent]
       Input.parseKeycode(event.keyCode).foreach { c => if (input == Some(c)) input= None }
     }
    document.addEventListener("keydown", kdHandler)
    document.addEventListener("keyup", kuHandler)
    return (Unit=> { document.removeEventListener("keydown", kdHandler);
                     document.removeEventListener("keyup", kuHandler)})
  }

  def viewControl () = {  
    val i= input
    println("Got input: "+ i)
    val s1= state.control(i)
    s1.render()
    state= s1
  }

  def exit(goodbye: String) = {
    // Remove all event handlers
    exitHandlers()
  }

  @JSExport
  def start() = {
    // wait for the document to be ready before starting the game
    document.onreadystatechange = (e: Event) => {
      if (document.readyState == "interactive")
        main(Array())
    }
    // register input handler
    val cancel= setInputHandler()
    // register main view-controller callback
    val ih= scala.scalajs.js.timers.setInterval(10)(viewControl)
    exitHandlers = (Unit=> { cancel()
                             scala.scalajs.js.timers.clearInterval(ih)})
  }

  def main(args: Array[String]) = {
    state= new State()
    state.init()
    println("Started!")
  }
}
