import org.scalajs.dom._

trait Input
case object L extends Input
case object R extends Input

object Input { 
  def parseKeycode(i: Int) : Option[Input] = i.asInstanceOf[Any] match {
    case 37 => Some(L)
    case 39 => Some(R)
    // case 81 => Some(Quit) 
    case _   => None
  }
}

case class Ball(x: Int, y: Int, vx: Int, vy: Int)

class State(paddle: Int, b: Ball, score: Int) {

  import Config._

  // Creates an initial state
  def this() = this((Config.rightBorder- Config.leftBorder)/2,
                     Ball((Config.rightBorder-Config.leftBorder)/2-1,
                          (Config.lowerBorder-Config.upperBorder)/2+1,
                           Config.speed, Config.speed), 0)

  def control(i: Option[Input]): State = {
    val p= i match {
      case Some(L) => if (paddle == leftBorder) paddle else paddle- 2*speed
      case Some(R) => if (paddle == rightBorder) paddle else paddle+ 2*speed
      case _ => paddle
    }
    val vx = if (b.x+ ballSize+ b.vx <= leftBorder ||
                 b.x+ ballSize+ b.vx >= rightBorder) -b.vx else b.vx
    val bounce= b.y+ ballSize+ b.vy >= lowerBorder && Math.abs(b.x- paddle) <= paddleSize
    val vy = if (b.y+ ballSize+ b.vy <= upperBorder ||bounce) -b.vy else b.vy
    new State(p, Ball(b.x+ vx, b.y+ vy, vx, vy), if (bounce) score+1 else score)
  }

  def render() = {
    val canvas = document.getElementById("canvas").asInstanceOf[html.Canvas]
    val context = canvas.getContext("2d").asInstanceOf[CanvasRenderingContext2D]
    // clear canvas
    context.fillStyle= "black"
    context.fillRect(0, 0, canvasWidth, canvasHeight)
    // draw walls
    context.fillStyle= "blue"
    context.fillRect(leftBorder- borderSize, upperBorder- borderSize, borderSize, lowerBorder- upperBorder+ borderSize)
    context.fillRect(leftBorder, upperBorder-borderSize, rightBorder-leftBorder, borderSize)
    context.fillRect(rightBorder, upperBorder- borderSize, borderSize, lowerBorder) 
    // Draw the ball
    context.fillStyle= "red"
    context.beginPath()
    context.arc(b.x, b.y, ballSize, 0, 2 * Math.PI, false)
    context.fill()
    // Draw the paddle
    context.fillStyle= "yellow"
    context.fillRect(paddle-paddleSize/2, lowerBorder, paddleSize, borderSize)
    // Score
    document.getElementById("score").innerHTML= score.toString
  }

  def init() = {   
    val canvas = document.getElementById("canvas").asInstanceOf[html.Canvas]
    canvas.width= canvasWidth
    canvas.height =canvasHeight
    // draw the rest
    this.render()
  }

}

object Config {

  // Configuration parameters
  val ballSize = 10    
  val paddleSize = 50
  val speed= 2

  val borderSize  = 10
  val upperBorder = 50
  val lowerBorder = 400
  val leftBorder  = 50
  val rightBorder = 700

  val canvasHeight = lowerBorder+ 50
  val canvasWidth = rightBorder+ 50

}
